/*
 * Copyright (c) 2003, Enrique Jose Garcia Cota
 * All rights reserved.
 *
 * This file is part of antlraux. antlraux is distributed under 
 * the BSD license. See the license terms in file license.txt.
 *
 */

package antlraux.context.asts;

import antlraux.util.LexInfoAST;
import antlraux.context.Scope;
import antlr.collections.AST;

/**
 * This AST implementation is very useful when modelling ASTs that store a
 * declaration that has a scope, like a class or method.
 * <p>
 * {@link ScopeAST} is capable of showing some additional information in
 * {@link #toString} if {@link #verboseStringConversion} is set to true using
 * {@link #setVerboseStringConversion(boolean)}:
 * 
 * <pre>
 *      ScopeAST ast = new ScopeAST(
 * </pre>
 * 
 * Warning: {@link TypeAST}, {@link ExpressionAST} and {@link ScopeAST} copy
 * first child and sibling with their only constructor. See their constructor's
 * comments for details.
 */
public class ScopeAST extends LexInfoAST {
    protected Scope scope;

    public ScopeAST() {
        super();
    }

    /**
     * This constructors calls {@link #initialize(AST)}, so it copies next
     * sibling and first child.
     */
    public ScopeAST(AST ast, Scope scope) {
        super();
        initialize(ast);
        setScope(scope);
    }

    public void setScope(Scope s) {
        scope = s;
    }

    public Scope getScope() {
        return scope;
    }

    private static boolean verboseStringConversion = false;

    /**
     * If set to true, this adds some additional information to the string
     * generated by {@link #toString()} (shows the AST's scope's name)
     */
    public static void setVerboseStringConversion(boolean b) {
        verboseStringConversion = b;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer(super.toString());

        if (verboseStringConversion) {
            sb.append(" (scope: ");
            if (scope != null)
                sb.append(scope.getName());
            else
                sb.append("<null>");
            sb.append(")");
        }

        return sb.toString();
    }

    /**
     * Warning: this implementation of initialize copies <tt>ast</tt>'s first
     * child and next sibling.
     */
    public void initialize(AST ast) {
        super.initialize(ast);
        this.setFirstChild(ast.getFirstChild());
        this.setNextSibling(ast.getNextSibling());

        if (ast instanceof ScopeAST) {
            this.scope = ((ScopeAST) ast).scope;
        }
    }
}